/**
 * 
 */
package org.gcube.portlets.user.workspace.server.util;

import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringUtils;
import org.gcube.application.framework.core.session.ASLSession;
import org.gcube.application.framework.core.session.SessionManager;
import org.gcube.applicationsupportlayer.social.ApplicationNotificationsManager;
import org.gcube.applicationsupportlayer.social.NotificationsManager;
import org.gcube.common.core.utils.logging.GCUBEClientLog;
import org.gcube.common.core.utils.logging.GCUBELog;
import org.gcube.portal.custom.scopemanager.scopehelper.ScopeHelper;
import org.gcube.portlets.user.homelibrary.home.HomeLibrary;
import org.gcube.portlets.user.homelibrary.home.exceptions.HomeNotFoundException;
import org.gcube.portlets.user.homelibrary.home.exceptions.InternalErrorException;
import org.gcube.portlets.user.homelibrary.home.workspace.Workspace;
import org.gcube.portlets.user.homelibrary.home.workspace.WorkspaceItem;
import org.gcube.portlets.user.homelibrary.home.workspace.WorkspaceItemType;
import org.gcube.portlets.user.homelibrary.home.workspace.exceptions.WorkspaceFolderNotFoundException;
import org.gcube.portlets.user.workspace.server.GWTWorkspaceBuilder;
import org.gcube.portlets.user.workspace.server.notifications.NotificationsProducer;
import org.gcube.portlets.user.workspace.server.util.scope.ScopeUtilFilter;


/**
 * @author Federico De Faveri defaveri@isti.cnr.it
 *
 */
public class Util {

	public static final String USERNAME_ATTRIBUTE = ScopeHelper.USERNAME_ATTRIBUTE;
	public static final String FOLDERIMPORTER_ATTRIBUTE = "FOLDER_IMPORTER";
	public static final String METADATACONVERTER_ATTRIBUTE = "METADATA_CONVERTER";
	public static final String WORKSPACE_EVENT_COLLECTOR_ATTRIBUTE = "EVENT_COLLECTOR";
	public static final String WORKSPACEBUILDER_ATTRIBUTE = "WORKSPACEBUILDER";
	public static final String NOTIFICATION_MANAGER = "NOTIFICATIONMANAGER";
	public static final String NOTIFICATION_PRODUCER = "NOTIFICATIONMANAGER";
	public static final String WS_RUN_IN_TEST_MODE = "WS_RUN_IN_TEST_MODE";
	public static final String WORKSPACE_SCOPE_UTIL = "WORKSPACE_SCOPE_UTIL";
	public static final String NOTIFICATION_PORTLET_CLASS_ID = "org.gcube.portlets.user.wsmail.server.WsMailServiceImpl"; //IN DEV
	
//	public static final String TEST_SCOPE = "/gcube/devsec";
//	public static final String TEST_USER = "pasquale.pagano";
	
	public static final String TEST_SCOPE = "/gcube";
//	public static final String TEST_USER = "federico.defaveri";
//	public static final String TEST_USER = "massimiliano.assante";
//	public static final String TEST_USER = "pasquale.pagano";
//	public static final String TEST_USER = "francesco.mangiacrapa";
//	public static final String TEST_USER_FULL_NAME = "Francesco Mangiacrapa";
//	public static final String TEST_USER = "aureliano.gentile";
	public static final String TEST_USER = "test.user";
	public static final String TEST_USER_FULL_NAME = "Test User";
	
	
	public static GCUBEClientLog defaultLogger = new GCUBEClientLog(Util.class);
	
	public static boolean withoutPortal = false;
	
	public static ASLSession getAslSession(HttpSession httpSession)
	{
		String sessionID = httpSession.getId();
		String user = (String) httpSession.getAttribute(USERNAME_ATTRIBUTE);
		ASLSession session;
		
		if (user == null) {
			
			user=TEST_USER;
			
			defaultLogger.warn("WORKSPACE PORTLET STARTING IN TEST MODE - NO USER FOUND - PORTLETS STARTING WITH FOLLOWING SETTINGS:");
			defaultLogger.warn("session id: "+sessionID);
			defaultLogger.warn("TEST_USER: "+user);
			defaultLogger.warn("TEST_SCOPE: "+TEST_SCOPE);
			defaultLogger.warn("USERNAME_ATTRIBUTE: "+USERNAME_ATTRIBUTE);
			session = SessionManager.getInstance().getASLSession(sessionID, user);
			session.setScope(TEST_SCOPE);
	
			//MANDATORY FOR SOCIAL LIBRARY
			session.setUserAvatarId(user + "Avatar");
			session.setUserFullName(TEST_USER_FULL_NAME);
			session.setUserEmailAddress(user + "@mail.test");
			
			//SET HTTP SESSION ATTRIBUTE
			httpSession.setAttribute(USERNAME_ATTRIBUTE, user);
			
			withoutPortal = true;
			
			return session;
			
		}else if(user.compareToIgnoreCase(TEST_USER)==0){
			
			withoutPortal = true;
		
		}else{
			
			withoutPortal = false;
		}
		
		return SessionManager.getInstance().getASLSession(sessionID, user);
	}
	

	public static Workspace getWorkspace(final HttpSession httpSession) throws InternalErrorException, HomeNotFoundException, WorkspaceFolderNotFoundException
	{
		final ASLSession session = getAslSession(httpSession);

		Workspace workspace = HomeLibrary.getUserWorkspace(session);
		
		final GCUBELog logger = getLogger(workspace);

		if (session.getAttribute(METADATACONVERTER_ATTRIBUTE) == null){

			logger.trace("Initializing the Metadata converter");

			Thread mcLoader = new Thread(new Runnable(){

				public void run() {
					try {
						MetadataConverter mc = new MetadataConverter(logger);
						mc.setup(session);
						session.setAttribute(METADATACONVERTER_ATTRIBUTE, mc);
					}catch (Exception e) {
						logger.error("Error initializing the Metadata Converter: "+e.getMessage());
					}
				}

			});

			mcLoader.start();
		}
		if (session.getAttribute(WORKSPACE_EVENT_COLLECTOR_ATTRIBUTE) == null){
			

			logger.trace("Initializing the event collector");

			//we prepare the event collector
			WorkspaceEventCollector eventCollector = new WorkspaceEventCollector(logger);
			workspace.addWorkspaceListener(eventCollector);
			session.setAttribute(WORKSPACE_EVENT_COLLECTOR_ATTRIBUTE, eventCollector);
		}
		
		if (session.getAttribute(WORKSPACEBUILDER_ATTRIBUTE) == null)
		{
			logger.trace("Initializing the workspace area builder");
			
			GWTWorkspaceBuilder builder = new GWTWorkspaceBuilder(logger);
			session.setAttribute(WORKSPACEBUILDER_ATTRIBUTE, builder);
		}

		return workspace;

	}

	public static GCUBELog getLogger(Workspace workspace)
	{
		/* TODO if (workspace instanceof PersistentFSWorkspace){
			PersistentFSWorkspace persistentFSWorkspace = (PersistentFSWorkspace) workspace;
			try {
				return persistentFSWorkspace.getWorkspacePortletLogger();
			} catch (InternalErrorException e) {
				e.printStackTrace();
			}
		}*/

		return defaultLogger;
	}

	public static MetadataConverter getMetadataConverter(GCUBELog logger, HttpSession httpSession)
	{
		MetadataConverter mc = (MetadataConverter) httpSession.getAttribute(METADATACONVERTER_ATTRIBUTE);

		if (mc==null)
		{

			mc = new MetadataConverter(logger);
			ASLSession session = getAslSession(httpSession);
			mc.setup(session);
			session.setAttribute(METADATACONVERTER_ATTRIBUTE, mc);
		}

		mc.isReady();

		return mc;
	}

	public static WorkspaceEventCollector getEventCollector(HttpSession httpSession)
	{
		ASLSession session = getAslSession(httpSession);
		return (WorkspaceEventCollector) session.getAttribute(Util.WORKSPACE_EVENT_COLLECTOR_ATTRIBUTE);
	}

	public static GWTWorkspaceBuilder getGWTWorkspaceBuilder(HttpSession httpSession)
	{
		ASLSession session = getAslSession(httpSession);
		return (GWTWorkspaceBuilder) session.getAttribute(Util.WORKSPACEBUILDER_ATTRIBUTE);
	}
	
	public static NotificationsManager getNotificationManager(ASLSession session)
	{
		
		NotificationsManager notifMng = (NotificationsManager) session.getAttribute(NOTIFICATION_MANAGER);
		
		if (notifMng == null) {
			defaultLogger.trace("Create new NotificationsManager for user: "+session.getUsername());
			notifMng = new ApplicationNotificationsManager(session, NOTIFICATION_PORTLET_CLASS_ID);
			session.setAttribute(NOTIFICATION_MANAGER, notifMng);
		}
		
		return notifMng;
	}
	
	public static NotificationsProducer getNotificationProducer(ASLSession session)
	{
		
		NotificationsProducer notifProducer = (NotificationsProducer) session.getAttribute(NOTIFICATION_PRODUCER);
		
		if (notifProducer == null) {
			defaultLogger.trace("Create new Notification Producer for user: "+session.getUsername());
			notifProducer = new NotificationsProducer(session);
			session.setAttribute(NOTIFICATION_PRODUCER, notifProducer);
		}
		
		return notifProducer;
	}

	public static String getUserId(HttpSession httpSession) {
		
		ASLSession session = getAslSession(httpSession);
		
		return session.getUsername();
	}
	
	public static boolean isVRE(ASLSession session){
		
		String currentScope = session.getScopeName();
		
		int slashCount = StringUtils.countMatches(currentScope, "/");
		
		if(slashCount < 3){
			defaultLogger.trace("currentScope is not VRE");
			return false;
		}
		
		defaultLogger.trace("currentScope is VRE");
		return true;
		
	}
	
	public static boolean isASharedFolder(WorkspaceItem wsItem){
		if(wsItem!=null)
			return wsItem.getType().equals(WorkspaceItemType.SHARED_FOLDER);
		return false;
	}
	
	public static ScopeUtilFilter getScopeUtilFilter(HttpSession httpSession){
		
		ASLSession session = getAslSession(httpSession);
		ScopeUtilFilter scopeUtil = null;
		try{
			scopeUtil = (ScopeUtilFilter) session.getAttribute(Util.WORKSPACE_SCOPE_UTIL);
			
			if(scopeUtil==null){
				scopeUtil = new ScopeUtilFilter(session.getScopeName());
				
			}
		}catch (Exception e) {
			defaultLogger.error("an error occurred in getscope filter "+e);
		}
		
		return scopeUtil;
	}

}
